﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using Microsoft.WindowsAzure.StorageClient;
using Microsoft.WindowsAzure;
using System.Threading;

namespace HeuristicLab.Services.Optimization.ControllerService.Logging {
  public class TableStorageTraceEntry : TableServiceEntity {
    public TableStorageTraceEntry(string message) {
      this.PartitionKey = "Trace";
      this.RowKey = Guid.NewGuid().ToString();
      this.Message = message;
    }

    public string Message { get; set; }
  };

  public class TableStorageTraceListener : TraceListener {

    private string traceTable;

    public string TraceTable {
      get {
        if (traceTable == null) {
          traceTable = CloudConfigurationManager.GetSetting("TraceTable");
        }
        return traceTable; 
      }
    }

    private bool traceEnabled = false;
    private DateTime? lastChecked = null;
    public bool TraceEnabled {
      get {
        if (!lastChecked.HasValue || (DateTime.Now - lastChecked.Value).TotalSeconds > 20) {
          lastChecked = DateTime.Now;
          traceEnabled = CloudConfigurationManager.GetSetting("TraceTableEnabled") != null && CloudConfigurationManager.GetSetting("TraceTableEnabled").ToLower() == "true";
        }
        return traceEnabled;
      }
    }

    private CloudStorageAccount account;
    private TableServiceContext context;
    private CloudTableClient client;
    private ThreadLocal<bool> doLog = new ThreadLocal<bool>();

    public TableServiceContext Context {
      get {
        if (account == null) {
          account = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting("StorageConnectionString"));
          client = account.CreateCloudTableClient();
          context = client.GetDataServiceContext();          
        }
        client.CreateTableIfNotExist(TraceTable);
        return context;
      }
    }


    public override void Write(string message) {
      if (!doLog.IsValueCreated) {
        doLog.Value = true;
      }

      // prevent stack overflows
      if (!doLog.Value) {
        return;
      }

      doLog.Value = false;
      if (TraceEnabled) {
        Context.AddObject(TraceTable, new TableStorageTraceEntry(message));
        Context.SaveChangesWithRetries();
      }
      doLog.Value = true;
    }

    public override void WriteLine(string message) {
      Write(message + "\n");
    }

    public override void TraceData(TraceEventCache eventCache, string source, TraceEventType eventType, int id, object data) {
      Write(data.ToString());
    }

    public override void TraceEvent(TraceEventCache eventCache, string source, TraceEventType eventType, int id, string message) {
      Write(message);
    }
  }
}
